#include <stdlib.h>
#include <iostream.h>
#include <string.h>
#include <ctype.h>

#include <options.h>

extern "C" {
   void exit(int);
   long  atol(const char *);
}

static const char * const optv[] = {
   "?|?",
   "H|help",
   "f?flags",
   "g+groups <name>",
   "c:count <int>",
   "s?str <string>",
   "x",
   " |hello",
   NULL
} ;

// I use the following routine when I see the -f option.
// It changes the opt_ctrls used by getopts() to take effect
// for the next option parsed. The -f option is used to test
// the Options::XXXXX flags and should be first on the command-line.
//
void
setflags(const char * flags_str, Options & opts) {
   if (flags_str && *flags_str) {
      unsigned  flags = opts.ctrls();
      if (::strchr(flags_str, '+'))  flags |= Options::PLUS;
      if (::strchr(flags_str, 'A'))  flags |= Options::ANYCASE;
      if (::strchr(flags_str, 'a'))  flags |= Options::ANYCASE;
      if (::strchr(flags_str, 'L'))  flags |= Options::LONG_ONLY;
      if (::strchr(flags_str, 'l'))  flags |= Options::LONG_ONLY;
      if (::strchr(flags_str, 'S'))  flags |= Options::SHORT_ONLY;
      if (::strchr(flags_str, 's'))  flags |= Options::SHORT_ONLY;
      if (::strchr(flags_str, 'Q'))  flags |= Options::QUIET;
      if (::strchr(flags_str, 'q'))  flags |= Options::QUIET;
      if (::strchr(flags_str, 'n'))  flags |= Options::NOGUESSING;
      if (::strchr(flags_str, 'N'))  flags |= Options::NOGUESSING;
      opts.ctrls(flags);
   }
}

main(int argc, char * argv[])
{
   int  optchar;
   const char * optarg;
   const char * str = "default_string";
   int  count = 0, xflag = 0, hello = 0;
   int  errors = 0;
   int  ngroups = 0;

   Options  opts(*argv, optv);
   OptArgvIter  iter(--argc, ++argv);

   while( optchar = opts(iter, optarg) ) {
      switch (optchar) {
      case '?' :
      case 'H' :
         opts.usage(cout, "files ...");
         ::exit(0);
         break;

      case 'f' : setflags(optarg, opts); break;

      case 'g' : ++ngroups; break;

      case 's' : str = optarg; break;

      case 'x' : ++xflag; break;

      case ' ' : ++hello; break;

      case 'c' :
         if (optarg == NULL) {
            ++errors;
         } else {
            count = (int) ::atol(optarg);
         }
         break;

      case Options::BADCHAR :  // bad option ("-%c", *optarg)
      case Options::BADKWD  :  // bad long-option ("--%s", optarg)
      case Options::AMBIGUOUS  :  // ambiguous long-option ("--%s", optarg)
      default :
         ++errors; break;
      } /*switch*/
   }

   int index = iter.index();

   if (errors || (index == argc)) {
      if (! errors) {
         cerr << opts.name() << ": no filenames given." << endl ;
      }
      opts.usage(cerr, "files ...");
      ::exit(1);
   }

   cout << "xflag=" << ((xflag) ? "ON"  : "OFF") << '\n'
        << "hello=" << ((hello) ? "YES" : "NO") << '\n'
        << "count=" << count << '\n'
        << "string=\"" << ((str) ? str : "No value given!") << "\"" << '\n'
        << "ngroups=" << ngroups << endl ;

   if (index < argc) {
      cout << "files=" ;
      for (int i = index ; i < argc ; i++) {
         cout << "\"" << argv[i] << "\" " ;
      }
      cout << endl ;
   }

   return  0;
}
